@@ -1,9 +1,6 @@
 /* -*- mode:c; coding:utf-8; tab-width:8; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-  Copyright (c) 2003-2004 MacUIM contributors and others
-  http://www.digital-genes.com/~yatsu/macuim/
-
-  All rights reserved.
+  Copyright (c) 2003-2005 MacUIM contributors, All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -33,6 +30,8 @@
 
 #define TARGET_API_MAC_CARBON 1
 
+#define OLDROUTINENAMES 1
+
 #include <Carbon/Carbon.h>
 
 #include "MUIM.h"
@@ -44,6 +43,8 @@ SegmentToHiliteRange(MUIMSessionHandle inHandle, TextRangeArrayPtr *inRange);
 static UInt32
 SegmentToOffset(MUIMSessionHandle inHandle);
 
+#pragma mark -
+
 /**
  * Create and sends an "update active input area" event to our client
  * application. We use the Carbon event manager to create a Carbon event,
@@ -177,7 +178,7 @@ MUIMUpdateActiveInputArea(MUIMSessionHandle inHandle, Boolean inFix)
       error = memFullErr;
   }
 
-#if DEBUG_INPUT_EVENT
+#if DEBUG_HILITERANGE
   {
     int i;
     for (i = 0; i < hiliteRangePtr->fNumOfRanges; i++) {
@@ -239,7 +240,7 @@ MUIMGetTSMStatus(MUIMSessionHandle inHandle, TSMStat *outStat)
   outStat->fReplyFont = 0;
   outStat->fReplyPointSize = 0;
   outStat->fReplyLineHeight = 0;
-
+  
   // create an event
   error = CreateEvent(NULL, kEventClassTextInput,
                       kEventTextInputOffsetToPos,
@@ -348,14 +349,15 @@ MUIMGetTSMStatus(MUIMSessionHandle inHandle, TSMStat *outStat)
   return error;
 }
 
+#pragma mark -
+
 static void
 SegmentToHiliteRange(MUIMSessionHandle inHandle, TextRangeArrayPtr *inRange)
 {
-  UInt32 i, pos = 0, j = 0;
-
-  (*inRange) = (TextRangeArrayPtr) NewPtrClear(sizeof(short) +
-                                               sizeof(TextRange) *
-                                               ((*inHandle)->fSegmentCount + 1));
+  UInt32 i, j = 0;
+  UInt32 pos = 0;
+  SInt32 caret = -1;
+  Boolean revExist = FALSE;
 
   /*
    * Uim attribute:
@@ -379,13 +381,23 @@ SegmentToHiliteRange(MUIMSessionHandle inHandle, TextRangeArrayPtr *inRange)
    * 7: kOutlineText  ?
    * 8: kSelectedText
    */
-
+  
+  (*inRange) = (TextRangeArrayPtr) NewPtrClear(sizeof(short) +
+                                               sizeof(TextRange) *
+                                               ((*inHandle)->fSegmentCount + 1));
+  
   for (i = 0; i < (*inHandle)->fSegmentCount; i++) {
-    // cursor
+#if DEBUG_HILITERANGE
+    DEBUG_PRINT("SegmentToHiliteRange() %lu pos=%lu len=%lu attr=%lu\n",
+                i, pos, (*inHandle)->fSegments[i].fLength,
+                (*inHandle)->fSegments[i].fAttr);
+#endif
+    // caret
     // string length is 0 or greaer than 0
     if ((*inHandle)->fSegments[i].fAttr & UPreeditAttr_Cursor) {
       (*inRange)->fRange[j].fStart = (*inRange)->fRange[j].fEnd = pos * sizeof(UniChar);
       (*inRange)->fRange[j].fHiliteStyle = kCaretPosition;
+      caret = j;
       j++;
     }
 
@@ -394,8 +406,10 @@ SegmentToHiliteRange(MUIMSessionHandle inHandle, TextRangeArrayPtr *inRange)
       pos += (*inHandle)->fSegments[i].fLength;
       (*inRange)->fRange[j].fEnd = pos * sizeof(UniChar);
 
-      if ((*inHandle)->fSegments[i].fAttr & UPreeditAttr_Reverse)
-          (*inRange)->fRange[j].fHiliteStyle = kSelectedConvertedText;
+      if ((*inHandle)->fSegments[i].fAttr & UPreeditAttr_Reverse) {
+        (*inRange)->fRange[j].fHiliteStyle = kSelectedConvertedText;
+        revExist = TRUE;
+      }
       else if ((*inHandle)->fSegments[i].fAttr & UPreeditAttr_UnderLine)
         (*inRange)->fRange[j].fHiliteStyle = kConvertedText;
       else
@@ -403,8 +417,14 @@ SegmentToHiliteRange(MUIMSessionHandle inHandle, TextRangeArrayPtr *inRange)
       j++;
     }
   }
-
   (*inRange)->fNumOfRanges = j;
+
+  if (!revExist) {
+    for (i = 0; i < (*inRange)->fNumOfRanges; i++) {
+      if ((*inRange)->fRange[i].fHiliteStyle == kConvertedText)
+        (*inRange)->fRange[i].fHiliteStyle = kSelectedRawText;
+    }
+  }
 }
 
 static UInt32
@@ -412,15 +432,6 @@ SegmentToOffset(MUIMSessionHandle inHandle)
 {
   UInt32 i, pos = 0;
 
-  /*
-  for (i = 0; i < (*inHandle)->fSegmentCount; i++) {
-    DEBUG_PRINT("segment %lu len=%lu buf='%s' attr=%lu\n", i,
-                (*inHandle)->fSegments[i].fLength,
-                (*inHandle)->fSegments[i].fBuffer,
-                (*inHandle)->fSegments[i].fAttr);
-  }
-  */
- 
   if ((*inHandle)->fSegmentCount <= 1)
     return 0;
   
