@@ -34,7 +34,7 @@ static ModeTipsController *sharedController;
 
 @implementation ModeTipsController
 
-+ (ModeTipsController *)sharedController
++ (id)sharedController
 {
   return sharedController;
 }
